@@ -7,7 +7,7 @@ from debputy.plugin.api import (
     PackageProcessingContext,
     define_debputy_plugin,
 )
-from debputy.util import _error
+from debputy.util import _error, PackageTypeSelector
 
 plugin_definition = define_debputy_plugin()
 
@@ -41,7 +41,8 @@ def _parse_numpy3_versions(versions_file: str) -> tuple[str, str]:
 
 @plugin_definition.metadata_or_maintscript_detector(
     # Probably not necessary, but this is the most faithful conversion
-    package_type=["deb", "udeb"],
+    package_types=PackageTypeSelector.DEB
+    | PackageTypeSelector.UDEB,
 )
 def numpy_depends(
     _unused: Any,
