@@ -1,5 +1,5 @@
 #include <stdio.h>
 #include "foo.h"
 void foo() {
-  printf("Hello world.\n");
+  printf("Printed from " __FILE__ ".\n");
 }
