@@ -66,7 +66,7 @@ except ImportError:
 from debputy.version import DEBPUTY_ROOT_DIR, DEBPUTY_PLUGIN_ROOT_DIR, version
 from debputy.filesystem_scan import (
     OSFSROOverlay,
-    FSRootDir,
+    InMemoryVirtualRootDir,
 )
 from debputy.plugin.api.impl_types import (
     DebputyPluginMetadata,
@@ -933,7 +933,7 @@ def assemble(
             continue
         # Ensure all fs's are read-only before we enable cross package checks.
         # This ensures that no metadata detector will never see a read-write FS
-        pkg_fs_root: "FSRootDir" = cast("FSRootDir", binary_data.fs_root)
+        pkg_fs_root: "InMemoryVirtualRootDir" = cast("FSRootDir", binary_data.fs_root)
         pkg_fs_root.is_read_write = False
 
     package_data_table.enable_cross_package_checks = True
