@@ -15,17 +15,10 @@ import textwrap
 from contextlib import ExitStack, suppress
 from tempfile import mkstemp
 from typing import (
-    List,
-    Optional,
-    Set,
-    Dict,
-    Tuple,
     Literal,
     TypeVar,
-    FrozenSet,
     cast,
     Any,
-    Union,
     AbstractSet,
     TYPE_CHECKING,
 )
@@ -38,7 +31,11 @@ from debputy._deb_options_profiles import DebBuildOptionsAndProfiles
 from debputy.architecture_support import DpkgArchitectureBuildProcessValuesTable
 from debputy.elf_util import find_all_elf_files, ELF_MAGIC
 from debputy.exceptions import DebputyDpkgGensymbolsError, PureVirtualPathError
-from debputy.filesystem_scan import FSPath, FSControlRootDir, VirtualPathBase
+from debputy.filesystem_scan import (
+    FSControlRootDir,
+    VirtualPathBase,
+    InMemoryVirtualPathBase,
+)
 from debputy.maintscript_snippet import (
     ALL_CONTROL_SCRIPTS,
     MaintscriptSnippetContainer,
@@ -136,7 +133,7 @@ def generate_md5sums_file(
 
 
 def install_or_generate_conffiles(
-    ctrl_root: FSPath | FSControlRootDir,
+    ctrl_root: InMemoryVirtualPathBase | FSControlRootDir,
     fs_root: VirtualPath,
     reserved_packager_provided_files: dict[str, list[PackagerProvidedFile]],
 ) -> None:
@@ -186,7 +183,7 @@ def _dpkg_perl_version(package: str) -> str:
 def handle_perl_code(
     dctrl_bin: BinaryPackage,
     dpkg_architecture_variables: DpkgArchitectureBuildProcessValuesTable,
-    fs_root: FSPath,
+    fs_root: InMemoryVirtualPathBase,
     substvars: FlushableSubstvars,
 ) -> None:
     perl_config_data = resolve_perl_config(dpkg_architecture_variables, dctrl_bin)
@@ -328,10 +325,15 @@ def _prune_dch_file(
                 continue
             block_date = block.date
             if block_date is None:
-                _error(f"The Debian changelog was missing date in sign off line")
-            entry_date = datetime.datetime.strptime(
-                block_date, "%a, %d %b %Y %H:%M:%S %z"
-            ).date()
+                _error("The Debian changelog was missing date in sign off line")
+            try:
+                entry_date = datetime.datetime.strptime(
+                    block_date, "%a, %d %b %Y %H:%M:%S %z"
+                ).date()
+            except ValueError:
+                _error(
+                    f"Invalid date in the changelog entry for version {block.version}: {block_date!r} (Expected format: 'Thu, 26 Feb 2026 00:00:00 +0000')"
+                )
             if (
                 trim
                 and entry_date < _DCH_PRUNE_CUT_OFF_DATE
@@ -464,7 +466,7 @@ def _detect_upstream_changelog(names: Iterable[str]) -> str | None:
 
 def install_upstream_changelog(
     dctrl_bin: BinaryPackage,
-    fs_root: FSPath,
+    fs_root: InMemoryVirtualPathBase,
     source_fs_root: VirtualPath,
 ) -> None:
     doc_dir = f"./usr/share/doc/{dctrl_bin.name}"
@@ -512,7 +514,7 @@ class _ElfInfo:
     fs_path: str
     is_stripped: bool | None = None
     build_id: str | None = None
-    dbgsym: FSPath | None = None
+    dbgsym: InMemoryVirtualPathBase | None = None
 
 
 def _elf_static_lib_walk_filter(
@@ -562,8 +564,8 @@ def _all_elf_files(fs_root: VirtualPath) -> Iterator[dict[str, _ElfInfo]]:
 
 
 def _find_all_static_libs(
-    fs_root: FSPath,
-) -> Iterator[FSPath]:
+    fs_root: InMemoryVirtualPathBase,
+) -> Iterator[InMemoryVirtualPathBase]:
     for path, children in fs_root.walk():
         # Matching the logic of dh_strip for now.
         if not _elf_static_lib_walk_filter(path, children):
@@ -592,7 +594,7 @@ def _find_all_static_libs(
 
 
 @contextlib.contextmanager
-def _all_static_libs(fs_root: FSPath) -> Iterator[list[str]]:
+def _all_static_libs(fs_root: InMemoryVirtualPathBase) -> Iterator[list[str]]:
     all_static_libs = list(_find_all_static_libs(fs_root))
     if not all_static_libs:
         yield []
@@ -629,8 +631,11 @@ def _resolve_build_ids(elf_info: dict[str, _ElfInfo]) -> None:
 
 
 def _make_debug_file(
-    objcopy: str, fs_path: str, build_id: str, dbgsym_fs_root: FSPath
-) -> FSPath:
+    objcopy: str,
+    fs_path: str,
+    build_id: str,
+    dbgsym_fs_root: InMemoryVirtualPathBase,
+) -> InMemoryVirtualPathBase:
     dbgsym_dirname = f"./usr/lib/debug/.build-id/{build_id[0:2]}/"
     dbgsym_basename = f"{build_id[2:]}.debug"
     dbgsym_dir = dbgsym_fs_root.mkdirs(dbgsym_dirname)
@@ -692,7 +697,9 @@ def _strip_binary(strip: str, options: list[str], paths: Iterable[str]) -> None:
             )
 
 
-def _attach_debug(objcopy: str, elf_binary: VirtualPath, dbgsym: FSPath) -> None:
+def _attach_debug(
+    objcopy: str, elf_binary: VirtualPath, dbgsym: InMemoryVirtualPathBase
+) -> None:
     dbgsym_fs_path: str
     with dbgsym.replace_fs_path_content() as dbgsym_fs_path:
         cmd = [objcopy, "--add-gnu-debuglink", dbgsym_fs_path, elf_binary.fs_path]
@@ -713,7 +720,7 @@ def _has_tool(tool: str) -> bool:
 
 def _run_dwz(
     dctrl: BinaryPackage,
-    dbgsym_fs_root: FSPath,
+    dbgsym_fs_root: InMemoryVirtualPathBase,
     unstripped_elf_info: list[_ElfInfo],
 ) -> None:
     if not unstripped_elf_info or dctrl.is_udeb or not _has_tool("dwz"):
@@ -755,7 +762,7 @@ def _run_dwz(
 
 def relocate_dwarves_into_dbgsym_packages(
     dctrl: BinaryPackage,
-    package_fs_root: FSPath,
+    package_fs_root: InMemoryVirtualPathBase,
     dbgsym_fs_root: VirtualPath,
     *,
     run_dwz: bool = False,
@@ -1431,7 +1438,7 @@ def _all_parent_directories_of(directories: Iterable[str]) -> set[str]:
 
 def _compute_multi_arch_for_arch_all_doc(
     binary_package: BinaryPackage,
-    fs_root: FSPath,
+    fs_root: InMemoryVirtualPathBase,
 ) -> str | None:
     if not binary_package.name.endswith(("-doc", "-docs")):
         # We limit by package name, since there are tricks involving a `Multi-Arch: no` depending on a
@@ -1453,7 +1460,7 @@ def _compute_multi_arch_for_arch_all_doc(
 
 
 def _any_unacceptable_paths(
-    fs_root: FSPath,
+    fs_root: InMemoryVirtualPathBase,
     *,
     acceptable_no_descend_paths: list[str] | AbstractSet[str] = frozenset(),
     acceptable_files: list[str] | AbstractSet[str] = frozenset(),
@@ -1475,7 +1482,7 @@ def _any_unacceptable_paths(
 def auto_compute_multi_arch(
     binary_package: BinaryPackage,
     control_output_dir: VirtualPath,
-    fs_root: FSPath,
+    fs_root: InMemoryVirtualPathBase,
 ) -> str | None:
     resolved_arch = binary_package.resolved_architecture
     if any(
@@ -1698,7 +1705,7 @@ def _generate_control_files(
     binary_package_data: "BinaryPackageData",
     package_state: "PackageTransformationDefinition",
     control_output_dir: FSControlRootDir,
-    fs_root: FSPath,
+    fs_root: InMemoryVirtualPathBase,
     substvars: FlushableSubstvars,
     dbgsym_root_fs: VirtualPath | None,
     dbgsym_build_ids: list[str] | None,
