@@ -436,17 +436,17 @@ class VirtualPathBase(VirtualPath, ABC):
 
         if not self.is_dir:
             raise TypeError(f"{self._orphan_safe_path()} is not a directory")
-        if any(self.iterdir()):
+        # No-op for the root directory. There is never a case where you want to delete this directory
+        # (and even if you could, debputy will need it for technical reasons, so the root dir stays)
+        if any(self.iterdir()) or self.is_root_dir():
             return
-        parent_dir = typing.cast("VirtualPathBase", assume_not_none(self.parent_dir))
+        parent_dir = self.parent_dir
 
         # Recursive does not matter; we already know the directory is empty.
         self.unlink()
 
-        # Note: The root dir must never be deleted. This works because when delegating it to the root
-        # directory, its implementation of this method is a no-op. If this is later rewritten to an
-        # inline loop (rather than recursion), be sure to preserve this feature.
-        parent_dir.prune_if_empty_dir()
+        if parent_dir:
+            parent_dir.prune_if_empty_dir()
 
     def _current_plugin(self) -> str:
         if self.is_detached:
@@ -533,11 +533,10 @@ class VirtualPathBase(VirtualPath, ABC):
                 yield fd
 
 
-class FSPath(VirtualPathBase, ABC):
+class InMemoryVirtualPathBase(VirtualPathBase, ABC):
     __slots__ = (
         "_basename",
         "_parent_dir",
-        "_children",
         "_path_cache",
         "_parent_path_cache",
         "_last_known_parent_path",
@@ -553,8 +552,7 @@ class FSPath(VirtualPathBase, ABC):
     def __init__(
         self,
         basename: str,
-        parent: Optional["FSPath"],
-        children: dict[str, "FSPath"] | None = None,
+        parent: Optional["InMemoryVirtualPathBase"],
         initial_mode: int | None = None,
         mtime: float | None = None,
         stat_cache: os.stat_result | None = None,
@@ -562,31 +560,21 @@ class FSPath(VirtualPathBase, ABC):
         self._basename = basename
         self._path_cache: str | None = None
         self._parent_path_cache: str | None = None
-        self._children = children
         self._last_known_parent_path: str | None = None
         self._mode = initial_mode
         self._mtime = mtime
         self._stat_cache = stat_cache
         self._metadata: dict[tuple[str, type[Any]], PathMetadataValue[Any]] = {}
+        # The `_owner` and `_group` is directly access outside the class via `tar_owner_info`
         self._owner = ROOT_DEFINITION
         self._group = ROOT_DEFINITION
 
         # The self._parent_dir = None is to create `_parent_dir` because the parent_dir setter calls
         # is_orphaned, which assumes self._parent_dir is an attribute.
-        self._parent_dir: ReferenceType["FSPath"] | None = None
+        self._parent_dir: ReferenceType["InMemoryVirtualPathBase"] | None = None
         if parent is not None:
             self.parent_dir = parent
 
-    def __repr__(self) -> str:
-        return (
-            f"{self.__class__.__name__}({self._orphan_safe_path()!r},"
-            f" is_file={self.is_file},"
-            f" is_dir={self.is_dir},"
-            f" is_symlink={self.is_symlink},"
-            f" has_fs_path={self.has_fs_path},"
-            f" children_len={len(self._children) if self._children else 0})"
-        )
-
     @property
     def name(self) -> str:
         return self._basename
@@ -606,11 +594,11 @@ class FSPath(VirtualPathBase, ABC):
         self._basename = new_name
         self.parent_dir = parent
 
-    def iterdir(self) -> Iterable["FSPath"]:
-        if self._children is not None:
-            yield from self._children.values()
+    # Overridden for new return type
+    def iterdir(self) -> Iterable["InMemoryVirtualPathBase"]:
+        raise NotImplementedError
 
-    def all_paths(self) -> Iterable["FSPath"]:
+    def all_paths(self) -> Iterable["InMemoryVirtualPathBase"]:
         yield self
         if not self.is_dir:
             return
@@ -622,7 +610,9 @@ class FSPath(VirtualPathBase, ABC):
             if current.is_dir and not current.is_detached:
                 stack.extend(sorted(current.iterdir(), key=by_basename, reverse=True))
 
-    def walk(self) -> Iterable[tuple["FSPath", list["FSPath"]]]:
+    def walk(
+        self,
+    ) -> Iterable[tuple["InMemoryVirtualPathBase", list["InMemoryVirtualPathBase"]]]:
         # FIXME: can this be more "os.walk"-like without making it harder to implement?
         if not self.is_dir:
             yield self, []
@@ -660,54 +650,23 @@ class FSPath(VirtualPathBase, ABC):
     # to using it)
     __iter__ = None
 
-    def __getitem__(self, key) -> "FSPath":
-        if self._children is None:
-            raise KeyError(
-                f"{key} (note: {self._orphan_safe_path()!r} has no children)"
-            )
-        if isinstance(key, FSPath):
-            key = key.name
-        return self._children[key]
+    # Overridden for new return type
+    def __getitem__(self, key: object) -> "InMemoryVirtualPathBase":
+        raise NotImplementedError
 
-    def __delitem__(self, key) -> None:
-        self._rw_check()
-        children = self._children
-        if children is None:
-            raise KeyError(key)
-        del children[key]
+    # Overridden for new return type
+    def get(self, key: str) -> "InMemoryVirtualPathBase | None":
+        return typing.cast("InMemoryVirtualPathBase | None", super().get(key))
 
-    def get(self, key: str) -> "Optional[FSPath]":
-        try:
-            return self[key]
-        except KeyError:
-            return None
+    def _add_child(self, child: "InMemoryVirtualPathBase") -> None:
+        raise TypeError(
+            f"{self._orphan_safe_path()!r} is not a directory (or did not implement this method)"
+        )
 
-    def __contains__(self, item: object) -> bool:
-        if isinstance(item, VirtualPath):
-            return item.parent_dir is self
-        if not isinstance(item, str):
-            return False
-        m = self.get(item)
-        return m is not None
-
-    def _add_child(self, child: "FSPath") -> None:
-        self._rw_check()
-        if not self.is_dir:
-            raise TypeError(f"{self._orphan_safe_path()!r} is not a directory")
-        if self._children is None:
-            self._children = {}
-
-        conflict_child = self.get(child.name)
-        if conflict_child is not None:
-            conflict_child.unlink(recursive=True)
-        self._children[child.name] = child
-
-    @property
-    def tar_path(self) -> str:
-        path = self.path
-        if self.is_dir:
-            return path + "/"
-        return path
+    def _remove_child(self, child: "InMemoryVirtualPathBase") -> None:
+        raise TypeError(
+            f"{self._orphan_safe_path()!r} is not a directory (or did not implement this method)"
+        )
 
     @property
     def path(self) -> str:
@@ -727,7 +686,7 @@ class FSPath(VirtualPathBase, ABC):
         return ret
 
     @property
-    def parent_dir(self) -> Optional["FSPath"]:
+    def parent_dir(self) -> Optional["InMemoryVirtualPathBase"]:
         p_ref = self._parent_dir
         p = p_ref() if p_ref is not None else None
         if p is None:
@@ -737,20 +696,19 @@ class FSPath(VirtualPathBase, ABC):
         return p
 
     @parent_dir.setter
-    def parent_dir(self, new_parent: Optional["FSPath"]) -> None:
+    def parent_dir(self, new_parent: Optional["InMemoryVirtualPathBase"]) -> None:
         self._rw_check()
         if new_parent is not None:
             if not new_parent.is_dir:
                 raise ValueError(
-                    f"The parent {new_parent._orphan_safe_path()} must be a directory"
+                    f"The parent {new_parent._orphan_safe_path()!r} must be a directory"
                 )
             new_parent._rw_check()
         old_parent = None
         self._last_known_parent_path = None
         if not self.is_detached:
             old_parent = self.parent_dir
-            old_parent_children = assume_not_none(assume_not_none(old_parent)._children)
-            del old_parent_children[self.name]
+            assume_not_none(old_parent)._remove_child(self)
         if new_parent is not None:
             self._parent_dir = ref(new_parent)
             new_parent._add_child(self)
@@ -791,7 +749,7 @@ class FSPath(VirtualPathBase, ABC):
         return st
 
     def _uncached_stat(self) -> os.stat_result:
-        return os.lstat(self.fs_path)
+        raise NotImplementedError
 
     @property
     def mode(self) -> int:
@@ -833,7 +791,7 @@ class FSPath(VirtualPathBase, ABC):
             self.mode |= min_bit
 
     def _resolve_initial_mtime(self) -> float:
-        return self.stat().st_mtime
+        raise NotImplementedError
 
     @property
     def mtime(self) -> float:
@@ -848,17 +806,6 @@ class FSPath(VirtualPathBase, ABC):
         self._rw_check()
         self._mtime = new_mtime
 
-    @property
-    def tar_owner_info(self) -> tuple[str, int, str, int]:
-        owner = self._owner
-        group = self._group
-        return (
-            owner.entity_name,
-            owner.entity_id,
-            group.entity_name,
-            group.entity_id,
-        )
-
     @property
     def _can_replace_inline(self) -> bool:
         return False
@@ -875,7 +822,7 @@ class FSPath(VirtualPathBase, ABC):
         # Special-case parameters that are not exposed in the API
         fs_basename_matters: bool = False,
         subdir_key: str | None = None,
-    ) -> Iterator["FSPath"]:
+    ) -> Iterator["InMemoryVirtualPathBase"]:
         if "/" in name or name in {".", ".."}:
             raise ValueError(f'Invalid file name: "{name}"')
         if not self.is_dir:
@@ -947,7 +894,7 @@ class FSPath(VirtualPathBase, ABC):
         require_copy_on_write: bool = True,
         follow_symlinks: bool = True,
         reference_path: VirtualPath | None = None,
-    ) -> "FSPath":
+    ) -> "InMemoryVirtualPathBase":
         if "/" in name or name in {".", ".."}:
             raise ValueError(f'Invalid file name: "{name}"')
         if not self.is_dir:
@@ -1004,7 +951,7 @@ class FSPath(VirtualPathBase, ABC):
         link_target: str,
         *,
         reference_path: VirtualPath | None = None,
-    ) -> "FSPath":
+    ) -> "InMemoryVirtualPathBase":
         if "/" in link_name or link_name in {".", ".."}:
             raise ValueError(
                 f'Invalid file name: "{link_name}" (it must be a valid basename)'
@@ -1033,7 +980,7 @@ class FSPath(VirtualPathBase, ABC):
         name: str,
         *,
         reference_path: VirtualPath | None = None,
-    ) -> "FSPath":
+    ) -> "InMemoryVirtualPathBase":
         if "/" in name or name in {".", ".."}:
             raise ValueError(
                 f'Invalid file name: "{name}" (it must be a valid basename)'
@@ -1054,8 +1001,8 @@ class FSPath(VirtualPathBase, ABC):
             raise ValueError(f"Path {existing.path} already exist")
         return VirtualDirectoryFSPath(name, self, reference_path=reference_path)
 
-    def mkdirs(self, path: str) -> "FSPath":
-        return cast("FSPath", super().mkdirs(path))
+    def mkdirs(self, path: str) -> "InMemoryVirtualPathBase":
+        return cast("InMemoryVirtualPathBase", super().mkdirs(path))
 
     @property
     def is_read_write(self) -> bool:
@@ -1079,7 +1026,7 @@ class FSPath(VirtualPathBase, ABC):
         """
         if self.is_detached:
             return
-        if not recursive and any(self.iterdir()):
+        if not recursive and self.is_dir and any(self.iterdir()):
             raise ValueError(
                 f'Refusing to unlink "{self.path}": The directory was not empty and recursive was False'
             )
@@ -1159,14 +1106,215 @@ class FSPath(VirtualPathBase, ABC):
         raise NotImplementedError
 
 
-class VirtualFSPathBase(FSPath, ABC):
+class InMemoryVirtualPath(InMemoryVirtualPathBase, ABC):
+    __slots__ = ("_children",)
+
+    def __init__(
+        self,
+        basename: str,
+        parent: Optional["InMemoryVirtualPathBase"],
+        children: dict[str, "InMemoryVirtualPathBase"] | None = None,
+        initial_mode: int | None = None,
+        mtime: float | None = None,
+        stat_cache: os.stat_result | None = None,
+    ) -> None:
+        super().__init__(
+            basename,
+            parent,
+            initial_mode,
+            mtime,
+            stat_cache,
+        )
+        self._children = children
+
+    def __repr__(self) -> str:
+        return (
+            f"{self.__class__.__name__}({self._orphan_safe_path()!r},"
+            f" is_file={self.is_file},"
+            f" is_dir={self.is_dir},"
+            f" is_symlink={self.is_symlink},"
+            f" has_fs_path={self.has_fs_path},"
+            f" children_len={len(self._children) if self._children else 0})"
+        )
+
+    def iterdir(self) -> Iterable["InMemoryVirtualPathBase"]:
+        if self._children is not None:
+            yield from self._children.values()
+
+    def __getitem__(self, key) -> "InMemoryVirtualPathBase":
+        if self._children is None:
+            raise KeyError(
+                f"{key} (note: {self._orphan_safe_path()!r} has no children)"
+            )
+        if isinstance(key, InMemoryVirtualPathBase):
+            key = key.name
+        return self._children[key]
+
+    def __delitem__(self, key) -> None:
+        self._rw_check()
+        children = self._children
+        if children is None:
+            raise KeyError(key)
+        del children[key]
+
+    def _remove_child(self, child: "InMemoryVirtualPathBase") -> None:
+        children = assume_not_none(self._children)
+        del children[child.name]
+
+    def _add_child(self, child: "InMemoryVirtualPathBase") -> None:
+        self._rw_check()
+        if not self.is_dir:
+            raise TypeError(f"{self._orphan_safe_path()!r} is not a directory")
+        if self._children is None:
+            self._children = {}
+
+        conflict_child = self.get(child.name)
+        if conflict_child is not None:
+            conflict_child.unlink(recursive=True)
+        self._children[child.name] = child
+
+    def _uncached_stat(self) -> os.stat_result:
+        return os.lstat(self.fs_path)
+
+    def _resolve_initial_mtime(self) -> float:
+        return self.stat().st_mtime
+
+
+class InMemoryOverlayFSDirectory(InMemoryVirtualPathBase):
+    __slots__ = (
+        "_underlying_directory",
+        "_virtual_children",
+        "_deleted_children",
+        "_has_underlying_read_children",
+    )
+
+    def __init__(
+        self,
+        underlying_directory: VirtualPath,
+        basename: str,
+        parent: Optional["InMemoryVirtualPathBase"],
+        initial_mode: int | None = None,
+        mtime: float | None = None,
+        stat_cache: os.stat_result | None = None,
+    ) -> None:
+        super().__init__(
+            basename,
+            parent,
+            initial_mode,
+            mtime,
+            stat_cache,
+        )
+        self._underlying_directory: VirtualPath = underlying_directory
+        self._virtual_children: dict[str, InMemoryVirtualPathBase] = {}
+        self._deleted_children = set[str]()
+        self._has_underlying_read_children = False
+
+    def __getitem__(self, key: object) -> "InMemoryVirtualPathBase":
+        if not isinstance(key, str) or key in self._deleted_children:
+            raise KeyError(key)
+        if c := self._virtual_children.get(key):
+            return c
+        if not self._ensure_read_underlying_children():
+            raise KeyError(key)
+        return self._virtual_children[key]
+
+    def _ensure_read_underlying_children(self) -> bool:
+        if self._has_underlying_read_children:
+            return False
+        for real_child in self._underlying_directory.iterdir():
+            basename = real_child.name
+            if basename in self._virtual_children or basename in self._deleted_children:
+                continue
+            if real_child.is_dir:
+                child = InMemoryOverlayFSDirectory(
+                    real_child,
+                    basename,
+                    self,
+                )
+            elif real_child.is_symlink:
+                child = SymlinkVirtualPath(
+                    basename,
+                    self,
+                    real_child.readlink(),
+                    reference_path=real_child,
+                )
+            else:
+                assert real_child.is_file and real_child.has_fs_path
+                child = FSBackedFilePath(
+                    basename,
+                    self,
+                    real_child.fs_path,
+                    reference_path=real_child,
+                )
+            self._virtual_children[child.name] = child
+
+    def _add_child(self, child: "InMemoryVirtualPathBase") -> None:
+        self._virtual_children[child.name] = child
+
+    def _remove_child(self, child: "InMemoryVirtualPathBase") -> None:
+        if child.name in self._deleted_children:
+            raise KeyError(child.name)
+        try:
+            del self._virtual_children[child.name]
+            deleted = True
+        except KeyError:
+            deleted = False
+        if not deleted and child.name not in self._underlying_directory:
+            raise KeyError(child.name)
+        self._deleted_children.add(child.name)
+
+    def _uncached_stat(self) -> os.stat_result:
+        try:
+            return self._underlying_directory.stat()  # type: ignore
+        except AttributeError:
+            raise PureVirtualPathError("The stat method has not been implemented")
+
+    def _resolve_initial_mtime(self) -> float:
+        return self._underlying_directory.mtime
+
+
+class InMemoryOverlayFSRootDirectory(InMemoryOverlayFSDirectory):
+    __slots__ = ("_fs_path", "_fs_read_write", "_plugin_context")
+
+    def __init__(self, underlying_directory: VirtualPath) -> None:
+        self._fs_read_write = True
+        super().__init__(
+            underlying_directory,
+            ".",
+            None,
+            initial_mode=0o755,
+        )
+        self._plugin_context = CurrentPluginContextManager("debputy")
+
+    def is_root_dir(self) -> bool:
+        return True
+
+    @property
+    def is_detached(self) -> bool:
+        return False
+
+    def unlink(self, *, recursive: bool = False) -> None:
+        # There is never a case where you want to delete this directory (and even if you could,
+        # debputy will need it for technical reasons, so the root dir stays)
+        raise TypeError("Cannot delete the root directory")
+
+    def _current_plugin(self) -> str:
+        return self._plugin_context.current_plugin_name
+
+    @contextlib.contextmanager
+    def change_plugin_context(self, new_plugin: str) -> Iterator[str]:
+        with self._plugin_context.change_plugin_context(new_plugin) as r:
+            yield r
+
+
+class VirtualFSPathBase(InMemoryVirtualPath, ABC):
     __slots__ = ()
 
     def __init__(
         self,
         basename: str,
-        parent: Optional["FSPath"],
-        children: dict[str, "FSPath"] | None = None,
+        parent: Optional["InMemoryVirtualPathBase"],
+        children: dict[str, "InMemoryVirtualPathBase"] | None = None,
         initial_mode: int | None = None,
         mtime: float | None = None,
         stat_cache: os.stat_result | None = None,
@@ -1205,7 +1353,7 @@ class VirtualFSPathBase(FSPath, ABC):
         return self.fs_path
 
 
-class FSRootDir(FSPath):
+class InMemoryVirtualRootDir(InMemoryVirtualPath):
     __slots__ = ("_fs_path", "_fs_read_write", "_plugin_context")
 
     def __init__(self, fs_path: str | None = None) -> None:
@@ -1231,11 +1379,11 @@ class FSRootDir(FSPath):
         return self.name
 
     @property
-    def parent_dir(self) -> Optional["FSPath"]:
+    def parent_dir(self) -> Optional["InMemoryVirtualPathBase"]:
         return None
 
     @parent_dir.setter
-    def parent_dir(self, new_parent: FSPath | None) -> None:
+    def parent_dir(self, new_parent: InMemoryVirtualPathBase | None) -> None:
         if new_parent is not None:
             raise ValueError("The root directory cannot become a non-root directory")
 
@@ -1287,11 +1435,6 @@ class FSRootDir(FSPath):
     def is_read_write(self, new_value: bool) -> None:
         self._fs_read_write = new_value
 
-    def prune_if_empty_dir(self) -> None:
-        # No-op for the root directory. There is never a case where you want to delete this directory
-        # (and even if you could, debputy will need it for technical reasons, so the root dir stays)
-        return
-
     def unlink(self, *, recursive: bool = False) -> None:
         # There is never a case where you want to delete this directory (and even if you could,
         # debputy will need it for technical reasons, so the root dir stays)
@@ -1312,7 +1455,7 @@ class VirtualPathWithReference(VirtualFSPathBase, ABC):
     def __init__(
         self,
         basename: str,
-        parent: FSPath,
+        parent: InMemoryVirtualPathBase,
         *,
         default_mode: int,
         reference_path: VirtualPath | None = None,
@@ -1389,7 +1532,7 @@ class VirtualDirectoryFSPath(VirtualPathWithReference):
     def __init__(
         self,
         basename: str,
-        parent: FSPath,
+        parent: InMemoryVirtualPathBase,
         *,
         reference_path: VirtualPath | None = None,
     ) -> None:
@@ -1425,7 +1568,7 @@ class SymlinkVirtualPath(VirtualPathWithReference):
     def __init__(
         self,
         basename: str,
-        parent_dir: FSPath,
+        parent_dir: InMemoryVirtualPathBase,
         link_target: str,
         *,
         reference_path: VirtualPath | None = None,
@@ -1464,7 +1607,7 @@ class FSBackedFilePath(VirtualPathWithReference):
     def __init__(
         self,
         basename: str,
-        parent_dir: FSPath,
+        parent_dir: InMemoryVirtualPathBase,
         fs_path: str,
         *,
         replaceable_inline: bool = False,
@@ -1527,7 +1670,7 @@ class FSBackedFilePath(VirtualPathWithReference):
 _SYMLINK_MODE = 0o777
 
 
-class VirtualTestPath(FSPath):
+class VirtualTestPath(InMemoryVirtualPath):
     __slots__ = (
         "_path_type",
         "_has_fs_path",
@@ -1540,7 +1683,7 @@ class VirtualTestPath(FSPath):
     def __init__(
         self,
         basename: str,
-        parent_dir: FSPath | None,
+        parent_dir: InMemoryVirtualPathBase | None,
         mode: int | None = None,
         mtime: float | None = None,
         is_dir: bool = False,
@@ -1992,7 +2135,7 @@ class OSFSROOverlay(OSFSOverlayBase["OSFSROOverlay"]):
             raise KeyError(key)
         if self._children is None:
             self._ensure_children_are_resolved()
-        if isinstance(key, FSPath):
+        if isinstance(key, InMemoryVirtualPathBase):
             key = key.name
         return assume_not_none(self._children)[key]
 
@@ -2137,7 +2280,7 @@ class OSFSControlPath(OSFSOverlayBase["OSFSControlPath"]):
         if not self.is_dir:
             raise KeyError(key)
         children = self._resolve_children(lambda n, fsp, p: OSFSControlPath(n, fsp, p))
-        if isinstance(key, FSPath):
+        if isinstance(key, InMemoryVirtualPathBase):
             key = key.name
         return children[key]
 
@@ -2310,9 +2453,9 @@ def as_path_defs(paths: Iterable[str | PathDef]) -> Iterable[PathDef]:
 def build_virtual_fs(
     paths: Iterable[str | PathDef],
     read_write_fs: bool = False,
-) -> "FSPath":
-    root_dir: FSRootDir | None = None
-    directories: dict[str, FSPath] = {}
+) -> "InMemoryVirtualPathBase":
+    root_dir: InMemoryVirtualRootDir | None = None
+    directories: dict[str, InMemoryVirtualPathBase] = {}
     non_directories = set()
 
     def _ensure_parent_dirs(p: str) -> None:
@@ -2344,7 +2487,7 @@ def build_virtual_fs(
             root_fs_path = None
             if path in (".", "./", "/"):
                 root_fs_path = path_def.fs_path
-            root_dir = FSRootDir(fs_path=root_fs_path)
+            root_dir = InMemoryVirtualRootDir(fs_path=root_fs_path)
             directories["."] = root_dir
 
         if path not in (".", "./", "/") and not path.startswith("./"):
@@ -2381,7 +2524,7 @@ def build_virtual_fs(
             non_directories.add(fs_path.path)
 
     if root_dir is None:
-        root_dir = FSRootDir()
+        root_dir = InMemoryVirtualRootDir()
 
     root_dir.is_read_write = read_write_fs
     return root_dir
