@@ -28,7 +28,12 @@ from .exceptions import (
     DebputySubstitutionError,
     DebputyRuntimeErrorWithPreamble,
 )
-from .filesystem_scan import FSPath, FSRootDir, OSFSROOverlay, FSControlRootDir
+from .filesystem_scan import (
+    InMemoryVirtualRootDir,
+    OSFSROOverlay,
+    FSControlRootDir,
+    InMemoryVirtualPathBase,
+)
 from .installations import (
     InstallRule,
     SourcePathMatcher,
@@ -86,6 +91,24 @@ from .yaml import MANIFEST_YAML
 from .yaml.compat import CommentedMap, CommentedSeq
 
 
+def tar_path(p: VirtualPath) -> str:
+    path = p.path
+    if p.is_dir:
+        return path + "/"
+    return path
+
+
+def tar_owner_info(path: InMemoryVirtualPathBase) -> tuple[str, int, str, int]:
+    owner = path._owner  # noqa
+    group = path._group  # noqa
+    return (
+        owner.entity_name,
+        owner.entity_id,
+        group.entity_name,
+        group.entity_id,
+    )
+
+
 class PathNotCoveredByInstallRulesError(DebputyRuntimeErrorWithPreamble):
 
     @property
@@ -110,7 +133,7 @@ class PathNotCoveredByInstallRulesError(DebputyRuntimeErrorWithPreamble):
 @dataclass(slots=True)
 class DbgsymInfo:
     binary_package: BinaryPackage
-    dbgsym_fs_root: FSPath
+    dbgsym_fs_root: InMemoryVirtualPathBase
     _dbgsym_root_fs: str | None
     dbgsym_ids: list[str]
     run_dwz: bool
@@ -138,7 +161,7 @@ class BinaryPackageData:
     source_package: SourcePackage
     binary_package: BinaryPackage
     binary_staging_root_dir: str
-    fs_root: FSPath
+    fs_root: InMemoryVirtualPathBase
     substvars: FlushableSubstvars
     package_metadata_context: PackageProcessingContextProvider
     ctrl_creator: BinaryCtrlAccessorProviderCreator
@@ -176,11 +199,11 @@ class PackageTransformationDefinition:
 
 
 def _path_to_tar_member(
-    path: FSPath,
+    path: InMemoryVirtualPathBase,
     clamp_mtime_to: int,
 ) -> TarMember:
     mtime = float(clamp_mtime_to)
-    owner, uid, group, gid = path.tar_owner_info
+    owner, uid, group, gid = tar_owner_info(path)
     mode = path.mode
 
     if path.has_fs_path:
@@ -198,7 +221,7 @@ def _path_to_tar_member(
             path.readlink(),
         )
         return TarMember.virtual_path(
-            path.tar_path,
+            tar_path(path),
             PathType.SYMLINK,
             mtime,
             link_target=link_target,
@@ -219,7 +242,7 @@ def _path_to_tar_member(
     if not path.has_fs_path:
         assert not path.is_file
         return TarMember.virtual_path(
-            path.tar_path,
+            tar_path(path),
             path_type,
             mtime,
             mode=mode,
@@ -228,9 +251,9 @@ def _path_to_tar_member(
             group=group,
             gid=gid,
         )
-    may_steal_fs_path = path._can_replace_inline
+    may_steal_fs_path = getattr(path, "_can_replace_inline", False)
     return TarMember.from_file(
-        path.tar_path,
+        tar_path(path),
         path.fs_path,
         mode=mode,
         uid=uid,
@@ -245,7 +268,7 @@ def _path_to_tar_member(
 
 
 def _generate_intermediate_manifest(
-    fs_root: FSPath,
+    fs_root: InMemoryVirtualPathBase,
     clamp_mtime_to: int,
 ) -> Iterable[TarMember]:
     symlinks = []
@@ -1442,7 +1465,7 @@ class HighLevelManifest:
 
             install_rule_context[package] = BinaryPackageInstallRuleContext(
                 dctrl_bin,
-                FSRootDir(),
+                InMemoryVirtualRootDir(),
                 doc_main_package,
             )
 
@@ -1551,7 +1574,7 @@ class HighLevelManifest:
                     "",
                     dh_dbgsym_root_fs,
                 )
-                dbgsym_root_fs = FSRootDir()
+                dbgsym_root_fs = InMemoryVirtualRootDir()
                 _install_everything_from_source_dir_if_present(
                     dctrl_bin,
                     self.substitution,
@@ -1573,7 +1596,7 @@ class HighLevelManifest:
             else:
                 dbgsym_info = DbgsymInfo(
                     dctrl_bin,
-                    FSRootDir(),
+                    InMemoryVirtualRootDir(),
                     None,
                     [],
                     False,
@@ -1612,7 +1635,7 @@ class HighLevelManifest:
     def apply_fs_transformations(
         self,
         package: str,
-        fs_root: FSPath,
+        fs_root: InMemoryVirtualPathBase,
     ) -> None:
         if package in self._used_for:
             raise ValueError(
@@ -1647,7 +1670,7 @@ class HighLevelManifest:
     def finalize_data_tar_contents(
         self,
         package: str,
-        fs_root: FSPath,
+        fs_root: InMemoryVirtualPathBase,
         clamp_mtime_to: int,
     ) -> IntermediateManifest:
         if package in self._used_for:
@@ -1675,7 +1698,7 @@ class HighLevelManifest:
     def apply_to_binary_staging_directory(
         self,
         package: str,
-        fs_root: FSPath,
+        fs_root: InMemoryVirtualPathBase,
         clamp_mtime_to: int,
     ) -> IntermediateManifest:
         self.apply_fs_transformations(package, fs_root)
