@@ -7,7 +7,7 @@ from typing import TYPE_CHECKING, cast, Any
 from collections.abc import Callable, Iterator, Sequence, Iterable, Mapping
 
 from debputy.exceptions import DebputyRuntimeError
-from debputy.filesystem_scan import FSPath
+from debputy.filesystem_scan import InMemoryVirtualPathBase
 from debputy.manifest_conditions import (
     ConditionContext,
     ManifestCondition,
@@ -96,7 +96,7 @@ class SearchDir:
 @dataclasses.dataclass(slots=True, frozen=True)
 class BinaryPackageInstallRuleContext:
     binary_package: BinaryPackage
-    fs_root: FSPath
+    fs_root: InMemoryVirtualPathBase
     doc_main_package: BinaryPackage
 
     def replace(self, **changes: Any) -> "BinaryPackageInstallRuleContext":
@@ -496,7 +496,7 @@ def _resolve_dest_paths(
     match: PathMatch,
     dest_paths: Sequence[tuple[str, bool]],
     install_context: "InstallRuleContext",
-) -> Sequence[tuple[str, "FSPath"]]:
+) -> Sequence[tuple[str, "InMemoryVirtualPathBase"]]:
     dest_and_roots = []
     for dest_path, dest_path_is_format in dest_paths:
         if dest_path_is_format:
@@ -532,8 +532,8 @@ def _resolve_matches(
     matches: list[PathMatch],
     dest_paths: Sequence[tuple[str, bool]] | Callable[[PathMatch], str],
     install_context: "InstallRuleContext",
-) -> Iterator[tuple[PathMatch, Sequence[tuple[str, "FSPath"]]]]:
-    dest_and_roots: Sequence[tuple[str, "FSPath"]]
+) -> Iterator[tuple[PathMatch, Sequence[tuple[str, "InMemoryVirtualPathBase"]]]]:
+    dest_and_roots: Sequence[tuple[str, "InMemoryVirtualPathBase"]]
     if callable(dest_paths):
         compute_dest_path = dest_paths
         for match in matches:
@@ -785,7 +785,7 @@ class InstallRule(DebputyDispatchableType):
     def _install_dir_recursively(
         self,
         path_matcher: SourcePathMatcher,
-        parent_dirs: Sequence[FSPath],
+        parent_dirs: Sequence[InMemoryVirtualPathBase],
         match: PathMatch,
         into: frozenset[BinaryPackage],
     ) -> None:
