@@ -38,7 +38,7 @@ from debputy.lsp.text_edit import (
     apply_text_edits,
     OverLappingTextEditException,
 )
-from debputy.lsp.vendoring._deb822_repro import (
+from debian._deb822_repro import (
     Deb822FileElement,
     Deb822ParagraphElement,
 )
