@@ -49,12 +49,12 @@ from debputy.lsp.diagnostics import (
     NATIVELY_LSP_SUPPORTED_SEVERITIES,
 )
 from debputy.lsp.spellchecking import Spellchecker, default_spellchecker
-from debputy.lsp.vendoring._deb822_repro import Deb822FileElement, parse_deb822_file
+from debian._deb822_repro import Deb822FileElement, parse_deb822_file
 from debputy.packages import SourcePackage, BinaryPackage
 from debputy.plugin.api.feature_set import PluginProvidedFeatureSet
 from debputy.plugin.api.spec import DebputyIntegrationMode
 from debputy.util import _warn, T
-from debputy.lsp.vendoring._deb822_repro.locatable import (
+from debian._deb822_repro.locatable import (
     Range as TERange,
     Position as TEPosition,
     Locatable,
@@ -381,7 +381,7 @@ class LintState:
         severity: LintSeverity,
         authority_reference: str,
         *,
-        quickfixes: list[dict] | None = None,
+        quickfixes: Sequence[Mapping] | None = None,
         tags: list[types.DiagnosticTag] | None = None,
         related_information: list[RelatedDiagnosticInformation] | None = None,
         diagnostic_applies_to_another_file: str | None = None,
