@@ -45,7 +45,7 @@ from debputy.lsp.maint_prefs import (
     EffectiveFormattingPreference,
 )
 from debputy.lsp.text_util import LintCapablePositionCodec
-from debputy.lsp.vendoring._deb822_repro import Deb822FileElement, parse_deb822_file
+from debian._deb822_repro import Deb822FileElement, parse_deb822_file
 from debputy.packages import (
     SourcePackage,
     BinaryPackage,
