@@ -17,7 +17,7 @@ from debputy.lsp.spellchecking import spellcheck_line
 from debputy.util import PKGVERSION_REGEX
 
 try:
-    from debputy.lsp.vendoring._deb822_repro.locatable import (
+    from debian._deb822_repro.locatable import (
         Position as TEPosition,
         Range as TERange,
     )
