@@ -69,10 +69,10 @@ from debputy.lsp.ref_models.deb822_reference_parse_models import (
     DCtrlSubstvar,
 )
 from debputy.lsp.text_util import markdown_urlify
-from debputy.lsp.vendoring._deb822_repro import (
+from debian._deb822_repro import (
     Deb822ParagraphElement,
 )
-from debputy.lsp.vendoring._deb822_repro.parsing import (
+from debian._deb822_repro.parsing import (
     Deb822KeyValuePairElement,
 )
 from debputy.lsprotocol.types import (
@@ -107,7 +107,7 @@ else:
     import debputy.lsprotocol.types as types
 
 try:
-    from debputy.lsp.vendoring._deb822_repro.locatable import (
+    from debian._deb822_repro.locatable import (
         Position as TEPosition,
         Range as TERange,
         START_POSITION,
@@ -142,11 +142,10 @@ class SubstvarMetadata:
 
     def render_metadata_fields(self) -> str:
         def_by = f"Defined by: {self.defined_by}"
-        dh_seq = (
-            f"DH Sequence: {self.dh_sequence}" if self.dh_sequence is not None else None
-        )
         doc_uris = self.doc_uris
-        parts = [def_by, dh_seq]
+        parts = [def_by]
+        if self.dh_sequence is not None:
+            parts.append(f"DH Sequence: {self.dh_sequence}")
         if doc_uris:
             if len(doc_uris) == 1:
                 parts.append(f"Documentation: {markdown_urlify(doc_uris[0])}")
@@ -470,7 +469,7 @@ def _custom_hover_relationship_field(
         )
     match = next(iter(PKGNAME_REGEX.finditer(word_at_position)), None)
     if match is None:
-        return
+        return None
     package = match.group()
     if state == "empty-cache":
         state = "loaded"
@@ -507,16 +506,17 @@ def _custom_hover_relationship_field(
     return None
 
 
-_CUSTOM_FIELD_HOVER = {
-    field: _custom_hover_relationship_field
-    for field in chain(
-        all_package_relationship_fields().values(),
-        all_source_relationship_fields().values(),
-    )
-    if field != "Provides"
-}
-
-_CUSTOM_FIELD_HOVER["Description"] = _custom_hover_description
+_CUSTOM_FIELD_HOVER = dict(
+    (
+        (field, _custom_hover_relationship_field)
+        for field in chain(
+            all_package_relationship_fields().values(),
+            all_source_relationship_fields().values(),
+        )
+        if field != "Provides"
+    ),
+    Description=_custom_hover_description,
+)
 
 
 def _custom_hover(
