@@ -48,7 +48,7 @@ else:
     import debputy.lsprotocol.types as types
 
 try:
-    from debputy.lsp.vendoring._deb822_repro.locatable import (
+    from debian._deb822_repro.locatable import (
         Position as TEPosition,
         Range as TERange,
         START_POSITION,
