@@ -46,7 +46,7 @@ from debputy.lsp.ref_models.deb822_reference_parse_models import (
     GENERIC_VARIABLE_REFERENCE_DATA_PARSER,
 )
 from debputy.lsp.text_util import markdown_urlify
-from debputy.lsp.vendoring._deb822_repro import (
+from debian._deb822_repro import (
     Deb822ParagraphElement,
 )
 from debputy.lsprotocol.types import (
@@ -65,11 +65,10 @@ from debputy.lsprotocol.types import (
     DocumentFormattingParams,
 )
 from debputy.manifest_parser.util import AttributePath
-from debputy.util import _info
 from debputy.yaml import MANIFEST_YAML
 
 try:
-    from debputy.lsp.vendoring._deb822_repro.locatable import (
+    from debian._deb822_repro.locatable import (
         Position as TEPosition,
         Range as TERange,
     )
