@@ -33,12 +33,12 @@ from debputy.lsp.text_util import (
     trim_end_of_line_whitespace,
     SemanticTokensState,
 )
-from debputy.lsp.vendoring._deb822_repro.locatable import (
+from debian._deb822_repro.locatable import (
     START_POSITION,
     Range as TERange,
     Position as TEPosition,
 )
-from debputy.lsp.vendoring._deb822_repro.parsing import (
+from debian._deb822_repro.parsing import (
     Deb822KeyValuePairElement,
     Deb822ParagraphElement,
     Deb822FileElement,
@@ -46,8 +46,8 @@ from debputy.lsp.vendoring._deb822_repro.parsing import (
     Deb822ParsedTokenList,
     Interpretation,
 )
-from debputy.lsp.vendoring._deb822_repro.tokens import tokenize_deb822_file, Deb822Token
-from debputy.lsp.vendoring._deb822_repro.types import TokenOrElement
+from debian._deb822_repro.tokens import Deb822Token
+from debian._deb822_repro.types import TokenOrElement
 from debputy.lsprotocol.types import (
     CompletionParams,
     CompletionList,
