@@ -18,7 +18,7 @@ from debputy.commands.debputy_cmd.output import OutputStyle
 from debputy.linting.lint_util import LintState
 from debputy.lsp.diagnostics import LintSeverity
 from debputy.lsp.quickfixes import propose_correct_text_quick_fix
-from debputy.lsp.vendoring._deb822_repro.locatable import (
+from debian._deb822_repro.locatable import (
     Position as TEPosition,
     Range as TERange,
 )
