@@ -11,7 +11,7 @@ from collections.abc import Mapping, Sequence, Callable, Iterable
 
 from debputy.lsp.named_styles import ALL_PUBLIC_NAMED_STYLES
 from debputy.lsp.ref_models.deb822_reference_parse_models import UsageHint
-from debputy.lsp.vendoring._deb822_repro import Deb822ParagraphElement
+from debian._deb822_repro import Deb822ParagraphElement
 
 if TYPE_CHECKING:
     import lsprotocol.types as types
