@@ -19,7 +19,7 @@ from collections.abc import Callable, Mapping, Iterable
 
 import debputy.lsp.data as data_dir
 from debputy.lsp.named_styles import ALL_PUBLIC_NAMED_STYLES
-from debputy.lsp.vendoring._deb822_repro.types import FormatterCallback
+from debian._deb822_repro.types import FormatterCallback
 from debputy.lsp.vendoring.wrap_and_sort import wrap_and_sort_formatter
 from debputy.packages import SourcePackage
 from debputy.util import _error
