@@ -23,7 +23,7 @@ else:
 
 
 try:
-    from debputy.lsp.vendoring._deb822_repro.locatable import (
+    from debian._deb822_repro.locatable import (
         Position as TEPosition,
         Range as TERange,
     )
