@@ -3,21 +3,17 @@ import sys
 from enum import Enum
 from typing import (
     NotRequired,
-    List,
     Any,
-    Optional,
     Literal,
-    Dict,
-    Union,
 )
 from collections.abc import Iterable
 
 from debputy.lsp.diagnostics import LintSeverity
-from debputy.lsp.vendoring._deb822_repro import (
+from debian._deb822_repro import (
     LIST_SPACE_SEPARATED_INTERPRETATION,
     LIST_COMMA_SEPARATED_INTERPRETATION,
 )
-from debputy.lsp.vendoring._deb822_repro.parsing import (
+from debian._deb822_repro.parsing import (
     LIST_UPLOADERS_INTERPRETATION,
     Deb822ParsedTokenList,
     Interpretation,
@@ -27,7 +23,7 @@ from debputy.lsp.vendoring._deb822_repro.parsing import (
     ListInterpretation,
     _parse_separator_list_value,
 )
-from debputy.lsp.vendoring._deb822_repro.tokens import (
+from debian._deb822_repro.tokens import (
     Deb822SpaceSeparatorToken,
     _value_line_tokenizer,
     _RE_WHITESPACE_SEPARATED_WORD_LIST,
