@@ -1,12 +1,13 @@
 # Code extracted from devscripts/wrap-and-sort with typing added
 import re
-from typing import Tuple, Iterable, Literal, Union
+from typing import Tuple, Literal, Union
+from collections.abc import Iterable
 
-from debputy.lsp.vendoring._deb822_repro.formatter import (
+from debian._deb822_repro.formatter import (
     one_value_per_line_formatter,
     FormatterContentToken,
 )
-from debputy.lsp.vendoring._deb822_repro.types import FormatterCallback
+from debian._deb822_repro.types import FormatterCallback
 
 PACKAGE_SORT = re.compile("^[a-z0-9]")
 BUILD_SYSTEMS = (
@@ -22,7 +23,7 @@ BUILD_SYSTEMS = (
 BUILD_SYSTEMS_RE = re.compile("(" + "|".join(BUILD_SYSTEMS) + ")")
 
 
-def _sort_packages_key(package: str) -> Tuple[int, str]:
+def _sort_packages_key(package: str) -> tuple[int, str]:
     if re.match(BUILD_SYSTEMS_RE, package):
         # Sort build-system related packages first.
         group_order = -1
@@ -40,7 +41,7 @@ def _emit_one_line_value(
     value_tokens: Iterable[FormatterContentToken],
     sep_token: FormatterContentToken,
     trailing_separator: bool,
-) -> Iterable[Union[str, FormatterContentToken]]:
+) -> Iterable[str | FormatterContentToken]:
     first_token = True
     yield " "
     for token in value_tokens:
@@ -56,7 +57,7 @@ def _emit_one_line_value(
 
 
 def wrap_and_sort_formatter(
-    indentation: Union[int, Literal["FIELD_NAME_LENGTH"]],
+    indentation: int | Literal["FIELD_NAME_LENGTH"],
     trailing_separator: bool = True,
     immediate_empty_line: bool = False,
     max_line_length_one_liner: int = 0,
