@@ -5,7 +5,7 @@ from collections.abc import Sequence
 
 from debputy.commands.debputy_cmd.context import CommandContext
 from debputy.deb_packaging_support import setup_control_files
-from debputy.filesystem_scan import FSRootDir
+from debputy.filesystem_scan import InMemoryVirtualRootDir
 from debputy.highlevel_manifest import HighLevelManifest
 from debputy.intermediate_manifest import IntermediateManifest
 from debputy.plugin.api.impl_types import PackageDataTable
@@ -112,7 +112,7 @@ def assemble_debs(
         ):
             # Discard the dbgsym part if it conflicts with a real package, or
             # we were asked not to build it.
-            dctrl_data.dbgsym_info.dbgsym_fs_root = FSRootDir()
+            dctrl_data.dbgsym_info.dbgsym_fs_root = InMemoryVirtualRootDir()
             dctrl_data.dbgsym_info.dbgsym_ids.clear()
         dbgsym_fs_root = dctrl_data.dbgsym_info.dbgsym_fs_root
         dbgsym_ids = dctrl_data.dbgsym_info.dbgsym_ids
