@@ -41,14 +41,14 @@ class SONAMEInfo:
     path: VirtualPath
     full_soname: str
     library: str
-    major_version: Optional[str]
+    major_version: str | None
 
 
 class ShlibsContent:
     def __init__(self) -> None:
-        self._deb_lines: List[str] = []
-        self._udeb_lines: List[str] = []
-        self._seen: Set[Tuple[str, str, str]] = set()
+        self._deb_lines: list[str] = []
+        self._udeb_lines: list[str] = []
+        self._seen: set[tuple[str, str, str]] = set()
 
     def add_library(
         self,
@@ -56,7 +56,7 @@ class ShlibsContent:
         major_version: str,
         dependency: str,
         *,
-        udeb_dependency: Optional[str] = None,
+        udeb_dependency: str | None = None,
     ) -> None:
         line = f"{library} {major_version} {dependency}\n"
         seen_key = ("deb", library, major_version)
@@ -100,7 +100,7 @@ class ShlibsContent:
 def extract_so_name(
     binary_package: BinaryPackage,
     path: VirtualPath,
-) -> Optional[SONAMEInfo]:
+) -> SONAMEInfo | None:
     objdump = binary_package.cross_command("objdump")
     output = subprocess.check_output([objdump, "-p", path.fs_path], encoding="utf-8")
     for r in SONAME_FORMATS:
@@ -118,7 +118,7 @@ def extract_so_name(
 def extract_soname_info(
     binary_package: BinaryPackage,
     fs_root: VirtualPath,
-) -> List[SONAMEInfo]:
+) -> list[SONAMEInfo]:
     so_files = elf_util.find_all_elf_files(
         fs_root,
         with_linking_type=ELF_LINKING_TYPE_DYNAMIC,
@@ -135,10 +135,10 @@ def extract_soname_info(
 def _compute_shlibs_content(
     binary_package: BinaryPackage,
     manifest: "HighLevelManifest",
-    soname_info_list: List[SONAMEInfo],
-    udeb_package_name: Optional[str],
+    soname_info_list: list[SONAMEInfo],
+    udeb_package_name: str | None,
     combined_shlibs: ShlibsContent,
-) -> Tuple[ShlibsContent, bool]:
+) -> tuple[ShlibsContent, bool]:
     shlibs_file_contents = ShlibsContent()
     unversioned_so_seen = False
     strict_version = manifest.package_state_for(binary_package.name).binary_version
@@ -181,8 +181,8 @@ def _compute_shlibs_content(
 
 def resolve_reserved_provided_file(
     basename: str,
-    reserved_packager_provided_files: Dict[str, List[PackagerProvidedFile]],
-) -> Optional[VirtualPath]:
+    reserved_packager_provided_files: dict[str, list[PackagerProvidedFile]],
+) -> VirtualPath | None:
     matches = reserved_packager_provided_files.get(basename)
     if matches is None:
         return None
@@ -195,11 +195,11 @@ def resolve_reserved_provided_file(
 def generate_shlib_dirs(
     pkg: BinaryPackage,
     root_dir: str,
-    soname_info_list: List[SONAMEInfo],
-    materialized_dirs: List[str],
+    soname_info_list: list[SONAMEInfo],
+    materialized_dirs: list[str],
 ) -> None:
-    dir_scanned: Dict[str, Dict[str, Set[str]]] = {}
-    dirs: Dict[str, str] = {}
+    dir_scanned: dict[str, dict[str, set[str]]] = {}
+    dirs: dict[str, str] = {}
     warn_dirs = {
         "/usr/lib",
         "/lib",
@@ -215,7 +215,7 @@ def generate_shlib_dirs(
         materialized_dir = dirs.get(abs_parent_path)
         if matches is None:
             matches = collections.defaultdict(set)
-            for child in p.iterdir:
+            for child in p.iterdir():
                 if not child.is_symlink:
                     continue
                 target = _normalize_link_target(child.readlink())
@@ -224,11 +224,12 @@ def generate_shlib_dirs(
                     continue
                 matches[target].add(child.name)
             dir_scanned[abs_parent_path] = matches
-        symlinks = matches.get(elf_binary.name)
-        if not symlinks:
+        symlinks = matches.get(elf_binary.name, set())
+        full_soname = soname_info.full_soname
+        if full_soname != elf_binary.name and full_soname not in symlinks:
             if abs_parent_path in warn_dirs:
                 _warn(
-                    f"Could not find any SO symlinks pointing to {elf_binary.absolute} in {pkg.name} !?"
+                    f"Could not find a {full_soname} symlink pointing to {elf_binary.absolute} in {pkg.name} !?"
                 )
             continue
         if materialized_dir is None:
@@ -236,7 +237,16 @@ def generate_shlib_dirs(
             materialized_dirs.append(materialized_dir)
             dirs[abs_parent_path] = materialized_dir
 
-        os.symlink(elf_binary.fs_path, os.path.join(materialized_dir, elf_binary.name))
+        # Ideally, we would symlink here, but unfortunately, we might have materialized and stolen the
+        # file by the time it is processed. So we copy to be safe.
+        subprocess.check_call(
+            [
+                "cp",
+                "--reflink=auto",
+                elf_binary.fs_path,
+                os.path.join(materialized_dir, elf_binary.name),
+            ]
+        )
         for link in symlinks:
             os.symlink(elf_binary.name, os.path.join(materialized_dir, link))
 
@@ -246,11 +256,11 @@ def compute_shlibs(
     control_output_dir: str,
     fs_root: VirtualPath,
     manifest: "HighLevelManifest",
-    udeb_package_name: Optional[str],
+    udeb_package_name: str | None,
     ctrl: BinaryCtrlAccessor,
-    reserved_packager_provided_files: Dict[str, List[PackagerProvidedFile]],
+    reserved_packager_provided_files: dict[str, list[PackagerProvidedFile]],
     combined_shlibs: ShlibsContent,
-) -> List[SONAMEInfo]:
+) -> list[SONAMEInfo]:
     assert not binary_package.is_udeb
     shlibs_file = os.path.join(control_output_dir, "shlibs")
     need_ldconfig = False
@@ -285,7 +295,7 @@ def compute_shlibs(
 
         if shlibs_file_contents:
             need_ldconfig = True
-            with open(shlibs_file, "wt", encoding="utf-8") as fd:
+            with open(shlibs_file, "w", encoding="utf-8") as fd:
                 shlibs_file_contents.write_to(fd)
 
     if symbols_template_file:
