@@ -58,6 +58,7 @@ from debputy.plugin.plugin_state import (
 )
 from debputy.substitution import VariableContext
 from debputy.util import (
+    _error,
     _normalize_path,
     package_cross_check_precheck,
     PackageTypeSelector,
@@ -566,7 +567,8 @@ class DispatchingObjectParser(
             path: AttributePath,
             parser_context: "ParserContextData",
         ) -> Mapping[str, Any]:
-            on_end_parse_step(name, value, path, parser_context)
+            if on_end_parse_step is not None:
+                on_end_parse_step(name, value, path, parser_context)
             return value
 
         if nested_in_package_context:
@@ -813,6 +815,8 @@ class PluginProvidedManifestVariable:
         value_resolver = self.variable_value
         if isinstance(value_resolver, str):
             res = value_resolver
+        elif value_resolver is None:
+            _error(f"variable {self.variable_name} is not set")
         else:
             res = value_resolver(variable_context)
         return res
