@@ -1,11 +1,9 @@
 import contextlib
 import dataclasses
 import io
-import os
 import tempfile
 import textwrap
 from typing import (
-    Optional,
     Literal,
     overload,
     TypeVar,
@@ -232,7 +230,7 @@ class DispatchablePluggableManifestRuleMetadata(Generic[DP]):
     dispatched_type: type[DP]
     unwrapped_constructor: "DIPHandler"
     expected_debputy_integration_mode: Container[DebputyIntegrationMode] | None = None
-    online_reference_documentation: Optional["ParserDocumentation"] = None
+    online_reference_documentation: "ParserDocumentation | None" = None
     apply_standard_attribute_documentation: bool = False
     source_format: Any | None = None
 
@@ -1091,7 +1089,6 @@ class VirtualPath:
         """
         raise NotImplementedError
 
-    @property
     def iterdir(self) -> Iterable["VirtualPath"]:
         """Returns an iterable that iterates over all children of this path
 
@@ -1100,7 +1097,7 @@ class VirtualPath:
         """
         raise NotImplementedError
 
-    def lookup(self, path: str) -> Optional["VirtualPath"]:
+    def lookup(self, path: str) -> "VirtualPath | None":
         """Perform a path lookup relative to this path
 
         As an example `doc_dir = fs_root.lookup('./usr/share/doc')`
@@ -1142,23 +1139,6 @@ class VirtualPath:
         """
         raise NotImplementedError
 
-    @property
-    def is_detached(self) -> bool:
-        """Returns True if this path is detached
-
-        Paths that are detached from the file system will not be present in the package and
-        most operations are unsafe on them. This usually only happens if the path or one of
-        its parent directories are unlinked (rm'ed) from the file system tree.
-
-        All paths are attached by default and will only become detached as a result of
-        an action to mutate the virtual file system.  Note that the file system may not
-        always be manipulated.
-
-        :return: True if the entry is detached. Detached entries should be discarded, so they
-        can be garbage collected.
-        """
-        raise NotImplementedError
-
     # The __getitem__ behaves like __getitem__ from Dict but __iter__ would ideally work like a Sequence.
     # However, that does not feel compatible, so lets force people to use .children instead for the Sequence
     # behavior to avoid surprises for now.
@@ -1183,7 +1163,7 @@ class VirtualPath:
         """
         raise NotImplementedError
 
-    def get(self, key: str) -> "Optional[VirtualPath]":
+    def get(self, key: str) -> "VirtualPath | None":
         """Lookup a (direct) child by name
 
         The following are the same:
@@ -1244,8 +1224,16 @@ class VirtualPath:
             return f"/{p}"
         return p
 
+    def is_root_dir(self) -> bool:
+        """Whether the current path is the root directory
+
+        :return: True if this path is the root directory. False otherwise
+        """
+        # The root directory is never detachable in the current setup
+        raise NotImplementedError
+
     @property
-    def parent_dir(self) -> Optional["VirtualPath"]:
+    def parent_dir(self) -> "VirtualPath | None":
         """The parent directory of this path
 
         Note this operation requires the path is "attached" (see `is_detached`).  All paths are attached
@@ -1334,8 +1322,8 @@ class VirtualPath:
 
     def chown(
         self,
-        owner: Optional["StaticFileSystemOwner"],
-        group: Optional["StaticFileSystemGroup"],
+        owner: "StaticFileSystemOwner | None",
+        group: "StaticFileSystemGroup | None",
     ) -> None:
         """Change the owner/group of this path
 
