@@ -16,7 +16,7 @@ from debian.debian_support import DpkgArchTable
 from debian.substvars import Substvars
 
 from debputy.architecture_support import DpkgArchitectureBuildProcessValuesTable
-from debputy.filesystem_scan import OSFSROOverlay, FSRootDir
+from debputy.filesystem_scan import OSFSROOverlay, InMemoryVirtualRootDir
 from debputy.packages import BinaryPackage, SourcePackage
 from debputy.plugin.api import (
     PluginInitializationEntryPoint,
@@ -387,7 +387,7 @@ def manifest_variable_resolution_context(
     debian_dir: VirtualPath | None = None,
 ) -> VariableContext:
     if debian_dir is None:
-        debian_dir = FSRootDir()
+        debian_dir = InMemoryVirtualRootDir()
 
     return VariableContext(debian_dir)
 
@@ -549,7 +549,7 @@ class ServiceRegistryTestImpl(ServiceRegistry[DSD]):
 @contextlib.contextmanager
 def _read_only_fs_root(fs_root: VirtualPath) -> Iterator[VirtualPath]:
     if fs_root.is_read_write:
-        assert isinstance(fs_root, FSRootDir)
+        assert isinstance(fs_root, InMemoryVirtualRootDir)
         fs_root.is_read_write = False
         yield fs_root
         fs_root.is_read_write = True
