@@ -1238,7 +1238,7 @@ class AutoconfBuildSystemRule(StepBasedBuildSystemRule):
             "--localstatedir=/var",
             "--disable-option-checking",
             silent_rules,
-            f"--libdir=${{prefix}}/{multi_arch}",
+            f"--libdir=${{prefix}}/lib/{multi_arch}",
             "--runstatedir=/run",
             "--disable-maintainer-mode",
             "--disable-dependency-tracking",
