@@ -26,6 +26,7 @@ from debputy.plugins.debputy.paths import (
     SYSTEMD_SYSUSERS_DIR,
 )
 from debputy.plugins.debputy.private_api import initialize_via_private_api
+from debputy.util import PackageTypeSelector
 
 
 def initialize_debputy_features(api: DebputyPluginInitializer) -> None:
@@ -119,7 +120,7 @@ def register_package_metadata_detectors(api: DebputyPluginInitializer) -> None:
     api.metadata_or_maintscript_detector(
         "built-using-relations",
         detect_built_using,
-        package_type=["deb", "udeb"],
+        package_types=PackageTypeSelector.DEB | PackageTypeSelector.UDEB,
     )
 
 
