@@ -6,7 +6,7 @@ import textwrap
 import typing
 from collections.abc import Iterable, Iterator, Mapping
 
-from debputy.bug1120283 import holds_with_profiles, holds_on_arch
+from debian.deb822 import PkgRelation
 from debputy.manifest_parser.util import AttributePath
 from debputy.packaging.alternatives import SYSTEM_DEFAULT_PATH_DIRS
 from debputy.plugin.api import (
@@ -592,11 +592,11 @@ def _built_using_enabled_matches(
         for deps, cond, attr in context.manifest_configuration(pkg, cls) or ():
             for is_first, relation in deps:
                 name = relation["name"]
-                if not holds_on_arch(relation, host, table):
+                if not PkgRelation.holds_on_arch(relation, host, table):
                     _debug_log(
                         f"{attr}: {name} in Build-Depends is disabled by host architecture {host}."
                     )
-                elif not holds_with_profiles(relation, profiles):
+                elif not PkgRelation.holds_with_profiles(relation, profiles):
                     _debug_log(
                         f"{attr}: {name} in Build-Depends is disabled by profiles {' '.join(profiles)}."
                     )
