@@ -1,24 +1,19 @@
-import dataclasses
 import os
 from typing import (
     NoReturn,
-    Optional,
-    Tuple,
-    List,
     Literal,
-    Dict,
     TypeVar,
     cast,
     final,
 )
-from collections.abc import Callable, Sequence
+from collections.abc import Sequence
 
 from debputy.exceptions import (
     DebputyRuntimeError,
     PureVirtualPathError,
     TestPathWithNonExistentFSPathError,
 )
-from debputy.filesystem_scan import FSPath
+from debputy.filesystem_scan import InMemoryVirtualPathBase
 from debputy.interpreter import (
     extract_shebang_interpreter_from_file,
 )
@@ -54,13 +49,6 @@ CreateSymlinkReplacementRule = Literal[
 VP = TypeVar("VP", bound=VirtualPath)
 
 
-@dataclasses.dataclass(frozen=True, slots=True)
-class PreProvidedExclusion:
-    tag: str
-    description: str
-    pruner: Callable[[FSPath], None]
-
-
 class TransformationRule(DebputyDispatchableType):
 
     __slots__ = ()
@@ -68,7 +56,7 @@ class TransformationRule(DebputyDispatchableType):
     @final
     def run_transform_file_system(
         self,
-        fs_root: FSPath,
+        fs_root: InMemoryVirtualPathBase,
         condition_context: ConditionContext,
     ) -> None:
         run_in_context_of_plugin_wrap_errors(
@@ -80,7 +68,7 @@ class TransformationRule(DebputyDispatchableType):
 
     def transform_file_system(
         self,
-        fs_root: FSPath,
+        fs_root: InMemoryVirtualPathBase,
         condition_context: ConditionContext,
     ) -> None:
         raise NotImplementedError
@@ -129,12 +117,14 @@ class TransformationRule(DebputyDispatchableType):
 
     def _ensure_is_directory(
         self,
-        fs_root: FSPath,
+        fs_root: InMemoryVirtualPathBase,
         path_to_directory: str,
         definition_source: str,
-    ) -> FSPath:
+    ) -> InMemoryVirtualPathBase:
         current, missing_parts = fs_root.attempt_lookup(path_to_directory)
-        current = self._fs_path_as_dir(cast("FSPath", current), definition_source)
+        current = self._fs_path_as_dir(
+            cast("InMemoryVirtualPathBase", current), definition_source
+        )
         if missing_parts:
             return current.mkdirs("/".join(missing_parts))
         return current
@@ -160,7 +150,7 @@ class RemoveTransformationRule(TransformationRule):
 
     def transform_file_system(
         self,
-        fs_root: FSPath,
+        fs_root: InMemoryVirtualPathBase,
         condition_context: ConditionContext,
     ) -> None:
         matched_any = False
@@ -208,7 +198,7 @@ class MoveTransformationRule(TransformationRule):
         self._condition = condition
 
     def transform_file_system(
-        self, fs_root: FSPath, condition_context: ConditionContext
+        self, fs_root: InMemoryVirtualPathBase, condition_context: ConditionContext
     ) -> None:
         if not self._evaluate_condition(self._condition, condition_context):
             return
@@ -313,7 +303,7 @@ class CreateSymlinkPathTransformationRule(TransformationRule):
 
     def transform_file_system(
         self,
-        fs_root: FSPath,
+        fs_root: InMemoryVirtualPathBase,
         condition_context: ConditionContext,
     ) -> None:
         if not self._evaluate_condition(self._condition, condition_context):
@@ -385,7 +375,7 @@ class CreateDirectoryTransformationRule(TransformationRule):
 
     def transform_file_system(
         self,
-        fs_root: FSPath,
+        fs_root: InMemoryVirtualPathBase,
         condition_context: ConditionContext,
     ) -> None:
         if not self._evaluate_condition(self._condition, condition_context):
@@ -492,7 +482,7 @@ class PathMetadataTransformationRule(TransformationRule):
 
     def transform_file_system(
         self,
-        fs_root: FSPath,
+        fs_root: InMemoryVirtualPathBase,
         condition_context: ConditionContext,
     ) -> None:
         if not self._evaluate_condition(self._condition, condition_context):
@@ -503,7 +493,7 @@ class PathMetadataTransformationRule(TransformationRule):
         capabilities = self._capabilities
         capability_mode = self._capability_mode
         definition_source = self._definition_source
-        d: list[FSPath] | None = [] if self._recursive else None
+        d: list[InMemoryVirtualPathBase] | None = [] if self._recursive else None
         needs_file_match = True
         if self._owner is not None or self._group is not None or self._mode is not None:
             needs_file_match = False
@@ -579,7 +569,7 @@ class ModeNormalizationTransformationRule(TransformationRule):
 
     def transform_file_system(
         self,
-        fs_root: FSPath,
+        fs_root: InMemoryVirtualPathBase,
         condition_context: ConditionContext,
     ) -> None:
         seen = set()
