@@ -9,7 +9,7 @@ from debputy.lsp.config.config_options import DCO_SPELLCHECK_COMMENTS
 from debputy.lsp.languages.lsp_debian_control import _lint_debian_control
 from debputy.lsp.lsp_debian_control_reference_data import CURRENT_STANDARDS_VERSION
 from debputy.lsp.maint_prefs import MaintainerPreferenceTable
-from debputy.lsp.vendoring._deb822_repro import parse_deb822_file
+from debian._deb822_repro import parse_deb822_file
 from debputy.packages import DctrlParser
 from debputy.plugin.api import virtual_path_def
 from debputy.plugin.api.feature_set import PluginProvidedFeatureSet
