@@ -5,7 +5,7 @@ import typing
 import pytest
 
 from conftest import ManifestParserFactory
-from debputy.filesystem_scan import FSRootDir
+from debputy.filesystem_scan import InMemoryVirtualRootDir
 from debputy.plugin.api import virtual_path_def
 from debputy.plugin.api.impl_types import (
     PackageProcessingContextProvider,
@@ -268,7 +268,7 @@ packages:
         )
         .run_metadata_detector(
             metadata_detector_id="built-using-relations",
-            fs_root=FSRootDir(),
+            fs_root=InMemoryVirtualRootDir(),
             context=PackageProcessingContextProvider(
                 manifest=manifest,
                 binary_package=binary_package,
