@@ -13,7 +13,7 @@ from debputy.deb_packaging_support import (
     generate_md5sums_file,
     install_or_generate_conffiles,
 )
-from debputy.filesystem_scan import build_virtual_fs, FSRootDir
+from debputy.filesystem_scan import build_virtual_fs, InMemoryVirtualRootDir
 from debputy.packages import BinaryPackage
 from debputy.plugin.api import virtual_path_def
 from debputy.plugin.api.spec import PathDef
@@ -511,7 +511,7 @@ def test_install_or_generate_conffiles(
     expected: list[str] | None,
 ) -> None:
     root_fs = build_virtual_fs(data_members)
-    ctrl_fs = FSRootDir()
+    ctrl_fs = InMemoryVirtualRootDir()
     install_or_generate_conffiles(ctrl_fs, root_fs, {})
     conffiles_member = ctrl_fs.get("conffiles")
     if expected is None:
