@@ -2,15 +2,15 @@ import textwrap
 
 import pytest
 
-from debputy import DEBPUTY_DOC_ROOT_DIR
 from debputy.exceptions import DebputySubstitutionError
 from debputy.highlevel_manifest_parser import YAMLManifestParser
 from debputy.manifest_parser.exceptions import ManifestParseException
 from debputy.plugin.api.test_api import build_virtual_file_system
+from debputy.version import debputy_doc_root_dir
 
 
 def normalize_doc_link(message) -> str:
-    return message.replace(DEBPUTY_DOC_ROOT_DIR, "{{DEBPUTY_DOC_ROOT_DIR}}")
+    return message.replace(debputy_doc_root_dir(), "{{DEBPUTY_DOC_ROOT_DIR}}")
 
 
 @pytest.fixture()
@@ -268,7 +268,7 @@ def test_create_symlinks_unknown_replacement_rule(manifest_parser_pkg_foo):
     msg = (
         'The attribute "packages.foo.transformations[0].create-symlink.replacement-rule [Line 8 column 32]"'
         " did not have a valid structure/type: Value (golf) must be one of the following literal values:"
-        ' "error-if-exists", "error-if-directory", "abort-on-non-empty-directory", "discard-existing"'
+        " `error-if-exists`, `error-if-directory`, `abort-on-non-empty-directory`, `discard-existing`"
     )
     assert normalize_doc_link(e_info.value.args[0]) == msg
 
@@ -495,8 +495,7 @@ def test_yaml_build_environment_default(
     manifest_parser_pkg_foo: YAMLManifestParser,
 ) -> None:
     content = textwrap.dedent(
-        (
-            """\
+        """\
 
         manifest-version: '0.1'
         default-build-environment:
@@ -506,7 +505,6 @@ def test_yaml_build_environment_default(
         # FIXME: we should not require an empty dict here
         - autoconf: {}
         """
-        )
     )
     manifest = manifest_parser_pkg_foo.parse_manifest(fd=content)
     envs = manifest.build_environments
@@ -522,8 +520,7 @@ def test_yaml_build_environments_no_default(
     manifest_parser_pkg_foo: YAMLManifestParser,
 ) -> None:
     content = textwrap.dedent(
-        (
-            f"""\
+        f"""\
 
         manifest-version: '0.1'
         build-environments:
@@ -534,7 +531,6 @@ def test_yaml_build_environments_no_default(
         - autoconf:
              environment: custom-env
         """
-        )
     )
     manifest = manifest_parser_pkg_foo.parse_manifest(fd=content)
     envs = manifest.build_environments
@@ -552,8 +548,7 @@ def test_yaml_build_environments_no_default_error(
     manifest_parser_pkg_foo: YAMLManifestParser,
 ) -> None:
     content = textwrap.dedent(
-        (
-            """\
+        """\
 
         manifest-version: '0.1'
         build-environments:
@@ -564,7 +559,6 @@ def test_yaml_build_environments_no_default_error(
         # FIXME: we should not require an empty dict here
         - autoconf: {}
         """
-        )
     )
     with pytest.raises(ManifestParseException) as e_info:
         manifest_parser_pkg_foo.parse_manifest(fd=content)
